#!/bin/bash

NAME=coreos
RAWDISK=$NAME.raw
DISKSIZE=10G
ROOTSIZE=9000MB
OSMIRROR=http://cdimage.ubuntu.com/
VERSION=14.04.1
ARCH=amd64
KMIRROR=http://security.ubuntu.com/ubuntu/pool/main/l/linux/
KERNEL=linux-image-3.13.0-24-generic_3.13.0-24.47_"$ARCH".deb
TMPDIR=/tmp/core

# Create and partition vdisk
fallocate -l $DISKSIZE $RAWDISK
parted -s $RAWDISK mklabel msdos
parted -s -a optimal $RAWDISK mkpart primary 0% $ROOTSIZE
parted -s $RAWDISK set 1 boot on
parted -s -a optimal $RAWDISK mkpart primary linux-swap $ROOTSIZE 100%

# Setup loop
losetup -f $RAWDISK
LOOP=$(losetup -ln -O name)
kpartx -a $LOOP
sleep 1

PART1=/dev/mapper/$(kpartx -l $LOOP | awk '/p1/ {print $1}')
PART2=/dev/mapper/$(kpartx -l $LOOP | awk '/p2/ {print $1}')

# Create filesystem
mkfs.ext4 -q $PART1
mkswap $PART2 &>/dev/null
ROOTUUID=$(blkid -s UUID -o value $PART1)
SWAPUUID=$(blkid -s UUID -o value $PART2)

# Download ubuntu-core and kernel
mkdir $TMPDIR
mount $PART1 $TMPDIR
cd /tmp
wget -q $OSMIRROR/ubuntu-core/releases/$VERSION/release/ubuntu-core-$VERSION-core-$ARCH.tar.gz
tar xzf /tmp/ubuntu-core-$VERSION-core-$ARCH.tar.gz -C /tmp/core
cd $TMPDIR/tmp
wget -q $KMIRROR/$KERNEL

# Create fstab 
echo "UUID=$ROOTUUID /    ext4 defaults 0 1" > /tmp/core/etc/fstab
echo "UUID=$SWAPUUID swap swap defaults 0 1" >> /tmp/core/etc/fstab

# Chrooted omgeving
mount -o bind /proc/ /tmp/core/proc/
mount -o bind /dev/ /tmp/core/dev
mount -o bind /sys  /tmp/core/sys

# Network
cp /etc/resolv.conf /tmp/core/etc/resolv.conf
echo "127.0.0.1 localhost.localdomain localhost" > /tmp/core/etc/hosts
echo "auto lo eth0" > /tmp/core/etc/network/interfaces
echo "iface eth0 inet dhcp" >> /tmp/core/etc/network/interfaces

cat <<- 'EOF1' > /tmp/core/tmp/ubuntu-core-chroot.sh
#!/bin/bash

export PATH=/sbin:/bin/:/usr/sbin:/usr/bin 
dpkg -i /tmp/linux-image*deb &>/dev/null

# Create user 
useradd -m ubuntu -p 7KF89fldqwHBI &>/dev/null
addgroup ubuntu adm &>/dev/null
addgroup ubuntu sudo &>/dev/null
EOF1

# Bootloader configuration 

cat <<- 'EOF2' > /tmp/core/boot/extlinux.conf 
DEFAULT ubuntu 
LABEL   ubuntu 
SAY     Boot Ubuntu-Core 
LINUX   /boot/vmlinuz-3.13.0-24-generic
APPEND  root=/dev/disk/by-uuid/UUID
INITRD  /boot/initrd.img-3.13.0-24-generic
EOF2

sed -i s,UUID,$ROOTUUID,g /tmp/core/boot/extlinux.conf
cp /tmp/core/boot/extlinux.conf /tmp/core/syslinux.conf

chmod +x /tmp/core/tmp/ubuntu-core-chroot.sh

# Start chroot
chroot /tmp/core /bin/bash -c "/tmp/ubuntu-core-chroot.sh"
rm /tmp/core/tmp/ubuntu-core-chroot.sh

# Umount everything
cd /
umount /tmp/core/dev/
umount /tmp/core/proc/
umount /tmp/core/sys/
 
# Bootloader install
extlinux --install /tmp/core/boot/ &>/dev/null
dd if=/usr/lib/syslinux/mbr.bin bs=440 conv=notrunc count=1 of=$LOOP &>/dev/null

# Cleanup
umount $TMPDIR
rm -rf $TMPDIR
rm /tmp/ubuntu*tar.gz
kpartx -d $LOOP
losetup -D

